/* Macintosh Routines in C to Communicate with a TI85/TI82 */

/* #includes */

#include <Devices.h>
#include <Serial.h>
#include <Types.h>
#include <GestaltEqu.h>
#include <Values.h>

/* Constants */

#define kBaseResID              128
#define kModemPort              1
#define kPrinterPort    0
#define kModemOut               "\p.AOut"
#define kModemIn                "\p.AIn"
#define kPrinterOut             "\p.BOut"
#define kPrinterIn              "\p.BIn"
#define fatal                   TRUE
#define notfatal                FALSE

/* Typedefs */

typedef unsigned char uchar;
typedef unsigned short uint;

/* Global Variables */

boolean gPortSelect /* 1 = Modem Port , 0 = Printer Port */
short   outChan /* modem output channel */
short   inChan /* modem input channel */

/* Prototypes */

void    initserial(void);
void    send(uchar);
uchar   recvbyt(void);
void    closeserial(void);
void    HandleError(Str255, OSErr, boolean);

/* main */

void    main(void)
{
        OSErr theErr;

        gPortSelect = kModemPort /* set modem port to be used - use only once */

        initserial();

        /* insert your code here */

        closeserial();
}

/* initserial */

void    initserial(void) /* Initialize the serial port of your choice */
{
        OSErr theErr;

        if (gPortSelect) /* for modem port */
        {
                if (noErr != (theErr=OpenDriver(kModemOut, &outChan))
                        HandleError("/pOpenDriver-Modem Port Output Error", theErr, 
fatal);

                if (noErr != (theErr=OpenDriver(kModemIn, &inChan))
                        HandleError("/pOpenDriver-Modem Port Input Error", theErr, 
fatal);
        }
        else /* for printer port */
        {
                if (noErr != (theErr=OpenDriver(kPrinterOut, &outChan))
                        HandleError("/pOpenDriver-Printer Port Output Error", 
theErr, fatal);
                if (noErr != (theErr=OpenDriver(kPrinterIn, &inChan))
                        HandleError("/pOpenDriver-Printer Port Input Error", 
theErr, fatal);
        };

        /* change paramaters if other communication settings are needed */

        if (noErr != (theErr=SerReset(outChan, baud9600+stop10+noParity+data8))
                        HandleError("/pSerReset-Output Error", theErr, fatal);
        if (noErr != (theErr=SerReset(inChan, baud9600+stop10+noParity+data8))
                        HandleError("/pSerReset-Input Error", theErr, fatal);

}

/* send */

void send(uchar b)
{
        OSErr theErr;
    long count;
    uchar buff[10];

    buff[0]= b;
    count = 1;

        if (gPortSelect) /* for modem port */
        {
                if (noErr != (theErr=FSWrite(aoutRefNum, &count, buff))
                        HandleError("/pFSWrite-Modem Port Output Error", theErr, 
fatal);
        }
        else /* for printer port */
        {
                if (noErr != (theErr=FSWrite(boutRefNum, &count, buff))
                        HandleError("/pFSWrite-Printer Port Output Error", theErr, 
fatal);

        };
}

/* recvbyt */

uchar recvbyt(void)
{
        OSErr theErr;
        long count;
        uchar buff[10];
        uchar byt;

        if (gPortSelect) /* for modem port */
        {
                do {
                        if (noErr != (theErr=SerGetBuffer(ainRefNum,&count))
                                HandleError("/pSerGetBuffer-Modem Port Input 
Error", theErr, fatal);

                        if (count==0)
                                break;
                        count = 1;

                        if (noErr != (theErr=FSRead(ainRefNum,&count, buff))
                                HandleError("/pFSRead-Modem Port Input Error", 
theErr, fatal);

                } while (1);
        }
        else /* for printer port */
        {
                do {
                        if (noErr != (theErr=SerGetBuffer(binRefNum,&count))
                                HandleError("/pSerGetBuffer-Printer Port Input 
Error", theErr, fatal);

                        if (count==0)
                                break;
                        count = 1;

                        if (noErr != (theErr=FSRead(binRefNum,&count, buff))
                                HandleError("/pFSRead-Printer Port Input Error", 
theErr, fatal);

                } while (1);

        };

        byt = 0x7F & buff[0]; /* removes the parity bit */

        return byt;
}

/* closeserial */

void    closeserial(void)
{
        OSErr theErr;

        if (gPortSelect) /* for modem port */
        {
                if (noErr != (theErr=CloseDriver(kModemOut))
                        HandleError("/pCloseDriver-Modem Port Output Error", 
theErr, fatal);

                if (noErr != (theErr=CloseDriver(kModemIn))
                        HandleError("/pCloseDriver-Modem Port Input Error", theErr, 
fatal);
        }
        else /* for printer port */
        {
                if (noErr != (theErr=CloseDriver(kPrinterOut))
                        HandleError("/pCloseDriver-Printer Port Output Error", 
theErr, fatal);
                if (noErr != (theErr=CloseDriver(kPrinterIn))
                        HandleError("/pCloseDriver-Printer Port Input Error", 
theErr, fatal);
        };
}


/* HandleError */

void    HandleError (Str255 errorString, OSErr theErr, Boolean fatal)

{
        Str255  errStringNum;

        NumToString ((long)theErr, errStringNum);

        ParamText (errorString, errStringNum,0,0);

        StopAlert (kBaseResID, NIL);

        if (fatal)
                ExitToShell();
}

